#include <CoreFoundation/CoreFoundation.h>
#include <CoreServices/CoreServices.h>
#include <QuickLook/QuickLook.h>

OSStatus GeneratePreviewForURL(void *thisInterface, QLPreviewRequestRef preview, CFURLRef url, CFStringRef contentTypeUTI, CFDictionaryRef options);
void CancelPreviewGeneration(void *thisInterface, QLPreviewRequestRef preview);

/* -----------------------------------------------------------------------------
   Generate a preview for file

   This function's job is to create preview for designated file
   ----------------------------------------------------------------------------- */

OSStatus GeneratePreviewForURL(void *thisInterface, QLPreviewRequestRef preview, CFURLRef url, CFStringRef contentTypeUTI, CFDictionaryRef options)
{
	Boolean previewGenerated = false;
	
	CFDataRef resourceData = NULL;
	SInt32 errorCode = 0;
	Boolean result = CFURLCreateDataAndPropertiesFromResource(kCFAllocatorDefault, url, &resourceData, NULL, NULL, &errorCode);
	if(result)
	{
		if(CFDataGetLength(resourceData) >= BPLIST_MAGIC_SIZE)
		{
			// Read the first bytes of the file
			uint8_t *headerBuffer = calloc(BPLIST_MAGIC_SIZE, 1);
			CFDataGetBytes(resourceData, CFRangeMake(0, BPLIST_MAGIC_SIZE), headerBuffer);
			
			if (memcmp(headerBuffer, BPLIST_MAGIC, BPLIST_MAGIC_SIZE) == 0)
			{
				// This is a binary property list. We create an XML representation.
				CFStringRef errorString = NULL;
				CFPropertyListRef propertyListRef = CFPropertyListCreateFromXMLData(kCFAllocatorDefault, resourceData, kCFPropertyListImmutable, &errorString);
				if(propertyListRef != NULL)
				{
					CFDataRef representedData = CFPropertyListCreateData(kCFAllocatorDefault, propertyListRef, kCFPropertyListXMLFormat_v1_0, 0, NULL);
					if(representedData != NULL)
					{
						// We have XML data to display
						QLPreviewRequestSetDataRepresentation(preview, representedData, kUTTypePlainText, NULL);
						previewGenerated = true;
						CFRelease(representedData);
					}
				}
				
				if(errorString)
				{
					CFRelease(errorString);
					errorString = NULL;
				}
				
				if(propertyListRef)
				{
					CFRelease(propertyListRef);
					propertyListRef = NULL;
				}
			}
			
			if(headerBuffer)
			{
				free(headerBuffer);
				headerBuffer = NULL;
			}
		}
	}
	
	if(!previewGenerated)
	{
		// Fallback for all cases except binary property list
		QLPreviewRequestSetURLRepresentation(preview, url, kUTTypePlainText, NULL);
	}
	
	if(resourceData)
	{
		CFRelease(resourceData);
		resourceData = NULL;
	}
	
    return noErr;
}

void CancelPreviewGeneration(void *thisInterface, QLPreviewRequestRef preview)
{
    // Implement only if supported
}
