#include <CoreFoundation/CoreFoundation.h>
#include <CoreServices/CoreServices.h>
#include <QuickLook/QuickLook.h>

OSStatus GenerateThumbnailForURL(void *thisInterface, QLThumbnailRequestRef thumbnail, CFURLRef url, CFStringRef contentTypeUTI, CFDictionaryRef options, CGSize maxSize);
void CancelThumbnailGeneration(void *thisInterface, QLThumbnailRequestRef thumbnail);

/* -----------------------------------------------------------------------------
    Generate a thumbnail for file

   This function's job is to create thumbnail for designated file as fast as possible
   ----------------------------------------------------------------------------- */

OSStatus GenerateThumbnailForURL(void *thisInterface, QLThumbnailRequestRef thumbnail, CFURLRef url, CFStringRef contentTypeUTI, CFDictionaryRef options, CGSize maxSize)
{
	Boolean previewGenerated = false;
	
	// Dictionary to add the 'STRING' badge on top of the thumbnail
	const void *theKeys[1] = { kQLThumbnailPropertyExtensionKey };
	const void *theValues[1] = { CFSTR("STRINGS") };
	CFDictionaryRef propertiesDict = CFDictionaryCreate(kCFAllocatorDefault, theKeys, theValues, 1, &kCFTypeDictionaryKeyCallBacks, &kCFTypeDictionaryValueCallBacks);
	
	CFDataRef resourceData = NULL;
	SInt32 errorCode = 0;
	Boolean result = CFURLCreateDataAndPropertiesFromResource(kCFAllocatorDefault, url, &resourceData, NULL, NULL, &errorCode);
	if(result)
	{
		if(CFDataGetLength(resourceData) >= BPLIST_MAGIC_SIZE)
		{
			// Read the first bytes of the file
			uint8_t *headerBuffer = calloc(BPLIST_MAGIC_SIZE, 1);
			CFDataGetBytes(resourceData, CFRangeMake(0, BPLIST_MAGIC_SIZE), headerBuffer);
			
			if (memcmp(headerBuffer, BPLIST_MAGIC, BPLIST_MAGIC_SIZE) == 0)
			{
				// This is a binary property list. We create an XML representation.
				CFStringRef errorString = NULL;
				CFPropertyListRef propertyListRef = CFPropertyListCreateFromXMLData(kCFAllocatorDefault, resourceData, kCFPropertyListImmutable, &errorString);
				if(propertyListRef != NULL)
				{
					CFDataRef representedData = CFPropertyListCreateData(kCFAllocatorDefault, propertyListRef, kCFPropertyListXMLFormat_v1_0, 0, NULL);
					if(representedData != NULL)
					{
						// We have XML data to display
						QLThumbnailRequestSetThumbnailWithDataRepresentation(thumbnail, representedData, kUTTypePlainText, NULL, propertiesDict);
						previewGenerated = true;
						CFRelease(representedData);
					}
				}
				
				if(errorString)
				{
					CFRelease(errorString);
					errorString = NULL;
				}
				
				if(propertyListRef)
				{
					CFRelease(propertyListRef);
					propertyListRef = NULL;
				}
			}
			
			if(headerBuffer)
			{
				free(headerBuffer);
				headerBuffer = NULL;
			}
		}
	}
	
	if(!previewGenerated)
	{
		// Fallback for all cases except binary property list
		QLThumbnailRequestSetThumbnailWithURLRepresentation(thumbnail, url, kUTTypePlainText, NULL, propertiesDict);
		
		
	}
	
	if(resourceData)
	{
		CFRelease(resourceData);
		resourceData = NULL;
	}
	
	if(propertiesDict)
	{
		CFRelease(propertiesDict);
		propertiesDict = NULL;
	}
    
    return noErr;
}

void CancelThumbnailGeneration(void *thisInterface, QLThumbnailRequestRef thumbnail)
{
    // Implement only if supported
}
